@@ -32,6 +32,7 @@
 #include "eel-types.h"
 #include <gtk/gtkmain.h>
 #include <gtk/gtkviewport.h>
+#include <gtk/gtkscrolledwindow.h>
 
 /* Arguments */
 enum
@@ -53,6 +54,8 @@ struct EelWrapTableDetails
 	EelJustification y_justification;
 	gboolean homogeneous;
 	GList *children;
+
+	gboolean is_scrolled : 1;
 };
 
 static void          eel_wrap_table_class_init           (EelWrapTableClass   *wrap_table_class);
@@ -99,8 +102,12 @@ static EelArtIPoint  wrap_table_get_scroll_offset        (const EelWrapTable  *w
 static GtkWidget *   wrap_table_find_child_at_point      (const EelWrapTable  *wrap_table,
 							  int                  x,
 							  int                  y);
+static gboolean      wrap_table_child_focus_in           (GtkWidget           *widget,
+							  GdkEventFocus       *event,
+							  gpointer             data);
 static void          wrap_table_layout                   (EelWrapTable        *wrap_table);
 
+
 EEL_CLASS_BOILERPLATE (EelWrapTable, eel_wrap_table, GTK_TYPE_CONTAINER)
 
 /* Class init methods */
@@ -404,6 +411,12 @@ eel_wrap_table_add (GtkContainer *container,
 		
 		gtk_widget_queue_resize (child);
 	}
+
+	if (wrap_table->details->is_scrolled) {
+		g_signal_connect (child, "focus_in_event", 
+				  G_CALLBACK (wrap_table_child_focus_in), 
+				  wrap_table);
+	}
 }
 
 static void
@@ -425,6 +438,13 @@ eel_wrap_table_remove (GtkContainer *container,
 	if (child_was_visible) {
 		gtk_widget_queue_resize (GTK_WIDGET (container));
 	}
+
+	if (wrap_table->details->is_scrolled) {
+		g_signal_handlers_disconnect_by_func (
+			child,
+			G_CALLBACK (wrap_table_child_focus_in), 
+			wrap_table);
+	}
 }
 
 static void
@@ -691,6 +711,20 @@ wrap_table_get_scroll_offset (const EelWrapTable *wrap_table)
 	return scroll_offset;
 }
 
+static gboolean
+wrap_table_child_focus_in (GtkWidget *widget, 
+			   GdkEventFocus *event, 
+			   gpointer data)
+{
+	g_return_val_if_fail (widget->parent && widget->parent->parent, FALSE);
+	g_return_val_if_fail (GTK_IS_VIEWPORT (widget->parent->parent), FALSE);
+
+	eel_gtk_viewport_scroll_to_rect (GTK_VIEWPORT (widget->parent->parent), 
+					 &widget->allocation);
+	
+	return FALSE;
+}
+
 static GtkWidget *
 wrap_table_find_child_at_point (const EelWrapTable *wrap_table,
 			       int x,
@@ -1000,3 +1034,38 @@ eel_wrap_table_get_num_children (const EelWrapTable *wrap_table)
 	return g_list_length (wrap_table->details->children);
 }
 
+GtkWidget *
+eel_scrolled_wrap_table_new (gboolean homogenous,
+			     GtkWidget **wrap_table_out)
+{
+	GtkWidget *scrolled_window;
+	GtkWidget *wrap_table;
+	GtkWidget *viewport;
+	
+	g_return_val_if_fail (wrap_table_out != NULL, NULL);
+	
+	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_AUTOMATIC);
+
+	viewport = gtk_viewport_new (gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)),
+				     gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window)));
+	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport),
+				      GTK_SHADOW_NONE);
+	
+	gtk_container_add (GTK_CONTAINER (scrolled_window),
+			   viewport);
+	
+	wrap_table = eel_wrap_table_new (homogenous);
+	gtk_container_add (GTK_CONTAINER (viewport),
+			   wrap_table);
+
+	gtk_widget_show (wrap_table);
+	gtk_widget_show (viewport);
+
+	EEL_WRAP_TABLE (wrap_table)->details->is_scrolled = 1;
+
+	*wrap_table_out = wrap_table;
+	return scrolled_window;
+}
